// ==========================================================================
// $Id: StopWatch.hh,v 1.14 2002/12/05 14:12:16 kaehler Exp $
// a stop watch for cpu time measurements
// ==========================================================================
// (C)opyright:
//
//   Max-Planck-Institut fuer Informatik
//   Im Stadtwald, 66111 Saarbruecken
//   http://www.mpi-sb.mpg.de
// 
// Creator: mcstammi (Marc Stamminger)
// Email:   mcstammi@mpi-sb.mpg.de
// ==========================================================================
// $Log: StopWatch.hh,v $
// Revision 1.14  2002/12/05 14:12:16  kaehler
// port to new g++ 3.2; tested with old 2.95 so far
//
// Revision 1.13  2001/04/26 19:36:01  brabec
// fixes for cygwin/vc port
//
// Revision 1.12  2000/10/10 08:34:24  htschirm
// changes tmk c/cxx macro definitions of ARCH_... and COMP_...
// moved Perf.*.* implementation files to perf_include/, so now
//   there is no need for EXCLUDE, and dependencies work right
//
// Revision 1.11  2000/05/25 08:32:49  mcstammi
// compatibility
//
// Revision 1.10  2000/02/23 16:27:02  anonymous
// some changes for the windows port
//
// Revision 1.9  2000/02/14 17:06:42  anonymous
// changes for Visual C++ (stefan)
//
// Revision 1.8  2000/02/10 12:59:36  mcstammi
// -LAND:std port II
//
// Revision 1.7  1999/12/07 18:17:47  brabec
// fixed StopWatch
//
// Revision 1.6  1999/12/07 09:57:13  brabec
// fixed for gcc 2.95.2 (linux)
//
// Revision 1.5  1999/10/28 09:51:00  brabec
// some changes for linux
//
// Revision 1.4  1999/09/21 20:05:47  brabec
// linux changes
//
// Revision 1.3  1999/09/21 18:25:55  brabec
// linux changes
//
// Revision 1.2  1999/08/05 07:17:29  mcstammi
// added #includes
//
// Revision 1.1  1999/07/09 07:43:21  mcstammi
// added Pool, StopWatch, Wrappers
//
// Revision 1.2  1999/06/28 11:31:41  htschirm
// fixed used namespaces
//
// ==========================================================================

#ifndef BASE_STOPWATCH_HH
#define BASE_STOPWATCH_HH

/*! \file  StopWatch.hh
    \brief a stop watch for cpu time measurements
 */

#include <iostream>
#include <limits.h>
#include <sys/types.h>

#if !defined(ARCH_MSWIN) && !defined(ARCH_CYGWIN)
#define USE_TIMES
#include <sys/times.h>
#ifndef COMP_GCC
#warning "n_comp_gcc"
#include <unistd.h>
#endif
#else
#error "here"
#include <time.h>
/* warper for Borland C++ */
inline float stopWatch_getTime() {
  time_t t;
  return float(time(&t));
}
#endif

#include <time.h>

namespace base { class StopWatch; }


/** \class StopWatch StopWatch.hh
  \brief a stop watch for cpu time measurements */
namespace base {
class StopWatch
{
  friend std::ostream& operator<<(std::ostream& ost, const StopWatch &w);
public:
  /** default constructor
      boolean parameter = true: print system time separately. */
  StopWatch(bool s = false)
  {
#ifdef USE_TIMES
    this->mClocksPerSec = sysconf(_SC_CLK_TCK);
#else
    // division by zero. intensively since if the routime comes here,
    // should be an error.
    this->mClocksPerSec = 0;
#endif
    system = s;
    reset();
  }

  /** run the stop watch; error, if watch runs already. */
  void run()
  {
    running = true;
    last_time = current_time(&last_stime);
  }

  /** stop the stop watch; error, if watch stopped already. */
  void stop()
  {
    long stime;
    acc_time += current_time(&stime) - last_time;
    acc_stime += stime - last_stime;
    running = false;
  }

  /** reset the watch to 0:00. */
  void reset()
  {
    running = false;
    acc_time = acc_stime = 0;    
  }
  
  /** give current time (in seconds); call only if watch is stopped. */
  float time() const
  {
#ifdef USE_TIMES
    return (float)acc_time/((float)(this->mClocksPerSec));
#else
    return (float)acc_time;
#endif
  }

  /** give current system time (in seconds); call only if watch is stopped. */
  float stime() const
  {
#ifdef USE_TIMES
    return (float)acc_stime/((float)(this->mClocksPerSec));
#else
    return (float)acc_stime;
#endif
  }      

  /** the current global time with respect to some fix point in the past. */
  static float globTime()
  {
#ifdef USE_TIMES
    struct tms t;
    return ((float)times(&t))/((float)(sysconf(_SC_CLK_TCK)));
#else
    return stopWatch_getTime();
#endif
  }
  
protected:
  /** return current process time;
      if pointer is passed, write system time to it. */
  static long
  current_time(long *stime = NULL)
  {
#ifdef USE_TIMES
    struct tms t;
    times(&t);
    if (stime)
      *stime = t.tms_stime;
    return t.tms_utime + t.tms_stime;
#else
    if(stime)
      *stime = 0;
    return stopWatch_getTime();
#endif
  }
  
  /** time output in nano, micro, milli or unit seconds. */
  static inline void
  print_time(std::ostream &ost, float t)
  {
    if (t < 1e-6)
      ost << t*1e9 << "ns";
    else if (t < 1e-3)
      ost << t*1e6 << "micros";
    else if (t < 1.)
      ost << t*1e3 << "ms";
    else
      ost << t << "s";
  }

  bool running,system;
  long acc_time,last_time;
  long acc_stime,last_stime;
  clock_t  mClocksPerSec;	// this depends on sysconf
};


inline
std::ostream& operator<<(std::ostream& ost, const StopWatch &w)
{
  w.print_time(ost,w.time());
  if (w.system)
  {
    ost << " (";
    w.print_time(ost,w.stime());
    ost << ")";
  }
  return ost;
}

}
#endif /* BASE_STOPWATCH_HH */
